unit MainForm;

{
  Main form for BoxCast LCL app.
  Copyright 2017 Phil Hess.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  LCLIntf, LCLType,
  VersInfoList, PrefsList, MbMap, NdfdLib;

type
  TMainForm = class(TForm)
    MainMenu       : TMainMenu;

    HelpMenu       : TMenuItem;
    HelpHelpCmd    : TMenuItem;
    HelpImproveCmd : TMenuItem;

    Map            : TMbMap;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

    procedure AboutCmdClick(Sender: TObject);

{$IFNDEF DARWIN}
    procedure FileExitCmdClick(Sender: TObject);
{$ENDIF}

    procedure HelpHelpCmdClick(Sender: TObject);
    procedure HelpImproveCmdClick(Sender: TObject);
    procedure MapDblClickMap(Sender   : TObject;
                             Latitude : Double;
                             Longitude: Double);

  private
{$IFDEF DARWIN}
    AppMenu        : TMenuItem;
    AppAboutCmd    : TMenuItem;
{$ENDIF}
{$IFNDEF DARWIN}
    FileMenu       : TMenuItem;
    FileExitCmd    : TMenuItem;
    HelpAboutCmd   : TMenuItem;
{$ENDIF}

    FmtSettings    : TFormatSettings;
    VersionList    : TVersionInfoList;
    ProgramName    : string;
    ProgramVersion : string;
    PrefsList      : TPreferencesList;
    Ndfd           : TNdfdLib;

    procedure SetupMenu;
    procedure LoadPreferences;
    procedure SavePreferences;
  end;

var
  MainFrm: TMainForm;


implementation

{$R *.lfm}


procedure TMainForm.FormCreate(Sender: TObject);
begin
  FmtSettings.DecimalSeparator := '.';  {Set before load preferences}

  VersionList := TVersionInfoList.Create;
  VersionList.DefaultLoad;
  ProgramName := VersionList.Values['ProductName'];
  ProgramVersion := VersionList.Values['ProductVersion'];

  PrefsList := TPreferencesList.Create;
  PrefsList.DefaultLoad(ProgramName);
  LoadPreferences;

  Application.Title := ProgramName;

  Ndfd := TNdfdLib.Create;

  SetupMenu;
end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Ndfd.Free;
  SavePreferences;
  PrefsList.Free;
  VersionList.Free;
end;


procedure TMainForm.AboutCmdClick(Sender: TObject);
begin
  MessageDlg(ProgramName + ', version ' + ProgramVersion + #10#10 +
             'Powered by Mapbox framework, version ' +
             FloatToStr(TMbMap.GetVersionNumber) + '.'#10#10 +
             'Weather forecast data retrieved by NDFD library, version ' +
             Ndfd.GetLibVersion + '.'#10#10 +
             'See help for more information.'#10#10 +
             VersionList.Values['LegalCopyright'],
             mtInformation, [mbOK], 0);
  //TODO: Proper About box.
end;


{$IFNDEF DARWIN}
procedure TMainForm.FileExitCmdClick(Sender: TObject);
begin
  Close;
end;
{$ENDIF}


procedure TMainForm.HelpHelpCmdClick(Sender: TObject);
begin
  MessageDlg('Quick help'#10#10 +
             'Double-click anywhere in the U.S. to see today''s forecast.'#10#10 +
             'To pan the map, click and hold the mouse down, then move the mouse.'#10#10 +
             'To zoom the map, use the zoom controls in the lower-right corner. ' +
             'You can also zoom by holding down the Shift key, then click and ' +
             'hold the mouse down and move the mouse.'#10#10 +
             'To tilt or rotate the map, hold down the option key, then ' + 
             'move the mouse up-down to tilt or left-right to rotate (or click ' +
             'on the compass above the zoom controls).', 
             mtInformation, [mbOK], 0);
  //TODO: Proper help.
end;


procedure TMainForm.HelpImproveCmdClick(Sender: TObject);
begin
  Map.GiveFeedback;
end;


procedure TMainForm.MapDblClickMap(Sender    : TObject;
                                   Latitude  : Double;
                                   Longitude : Double);
begin
  if not Ndfd.LoadForecast(FloatToStr(Latitude, FmtSettings),
                           FloatToStr(Longitude, FmtSettings),
                           False) then
    begin
    MessageDlg('Unable to retrieve weather forecast data.', mtError, [mbOK], 0);
    Exit;
    end;

  Map.AddPointAnnotation(Latitude, Longitude,
                         'Today''s forecast',
                         'High ' + Ndfd.GetMaxTemp(1) +
                         ', Low ' + Ndfd.GetMinTemp(1));
end;


procedure TMainForm.SetupMenu;
begin
{$IFDEF DARWIN}
  AppMenu := TMenuItem.Create(Self);  {Application menu}
  AppMenu.Caption := #$EF#$A3#$BF;  {Unicode Apple logo char}
  MainMenu.Items.Insert(0, AppMenu);

  AppAboutCmd := TMenuItem.Create(Self);
  AppAboutCmd.Caption := 'About ' + ProgramName;
  AppAboutCmd.OnClick := {$IFDEF FPC_OBJFPC}@{$ENDIF}AboutCmdClick;
  AppMenu.Add(AppAboutCmd);  {Add About as item in application menu}
{$ENDIF}

{$IFNDEF DARWIN}
  FileMenu := TMenuItem.Create(Self);
  FileMenu.Caption := 'File';
  MainMenu.Items.Insert(0, FileMenu);

  FileExitCmd := TMenuItem.Create(Self);
  FileExitCmd.Caption := 'Exit';
  FileExitCmd.OnClick := {$IFDEF FPC_OBJFPC}@{$ENDIF}FileExitCmdClick;
  FileMenu.Add(FileExitCmd);

  HelpAboutCmd := TMenuItem.Create(Self);
  HelpAboutCmd.Caption := 'About ' + ProgramName;
  HelpAboutCmd.OnClick := {$IFDEF FPC_OBJFPC}@{$ENDIF}AboutCmdClick;
  HelpMenu.Add(HelpAboutCmd);
{$ENDIF}

  HelpHelpCmd.Caption := ProgramName + ' Help';
end;


procedure TMainForm.LoadPreferences;
begin
  Left := StrToIntDef(PrefsList.Values['Startup', 'Left'], Left);
  Top := StrToIntDef(PrefsList.Values['Startup', 'Top'], Top);
  Width := StrToIntDef(PrefsList.Values['Startup','Width'], Width);
  Height := StrToIntDef(PrefsList.Values['Startup', 'Height'], Height);
  if PrefsList.Values['Startup', 'Maximized'] = 'True' then
    begin
    Position := poDesigned;
    WindowState := wsMaximized;
    end;

  Map.Latitude := StrToFloatDef(PrefsList.Values['Startup', 'Latitude'],
                                Map.Latitude, FmtSettings);
  Map.Longitude := StrToFloatDef(PrefsList.Values['Startup', 'Longitude'],
                                 Map.Longitude, FmtSettings);
  Map.ZoomLevel := StrToFloatDef(PrefsList.Values['Startup', 'ZoomLevel'],
                                 Map.ZoomLevel, FmtSettings);
  Map.Direction := StrToFloatDef(PrefsList.Values['Startup', 'Direction'],
                                 Map.Direction, FmtSettings);
end;


procedure TMainForm.SavePreferences;
var
  IsMaxStr : string;
begin
  PrefsList.Values['Startup', 'Left'] := IntToStr(Left);
  PrefsList.Values['Startup', 'Top'] := IntToStr(Top);
  PrefsList.Values['Startup', 'Width'] := IntToStr(Width);
  PrefsList.Values['Startup', 'Height'] := IntToStr(Height);
  IsMaxStr := 'False';
  if WindowState = wsMaximized then
    IsMaxStr := 'True';
  PrefsList.Values['Startup', 'Maximized'] := IsMaxStr;

  PrefsList.Values['Startup', 'Latitude'] := FloatToStr(Map.Latitude, FmtSettings);
  PrefsList.Values['Startup', 'Longitude'] := FloatToStr(Map.Longitude, FmtSettings);
  PrefsList.Values['Startup', 'ZoomLevel'] := FloatToStr(Map.ZoomLevel, FmtSettings);
  PrefsList.Values['Startup', 'Direction'] := FloatToStr(Map.Direction, FmtSettings);
end;


end.
