unit CocoaMbMap;

{
  LCL Mapbox map control.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).

  Mapbox macOS SDK: https://mapbox.github.io/mapbox-gl-native/macos
}

{$mode objfpc}{$H+}
{$modeswitch objectivec1}

interface

uses
  SysUtils,
  CocoaAll,  //use instead of AppKit for Cocoa widgetset's NSView category methods
  CoreLocation,
  Mapbox,
  LCLType,
  Controls,
  CocoaPrivate,
  WSMbMap,
  CocoaMbMapDelegate;

type
  TCocoaWSMbMap = class(TWSMbMap)
  published
    class function CreateHandle(const AWinControl : TWinControl;
                                const AParams     : TCreateParams): TLCLIntfHandle; override;
    class procedure DestroyHandle(const AWinControl : TWinControl); override;
    class function GetVersionNumber : Double;
    class procedure SetAccessToken(const AAccessToken : string); override;
    class procedure AddPointAnnotation(const AWinControl : TWinControl;
                                             ALatitude   : Double;
                                             ALongitude  : Double;
                                       const ATitle      : string;
                                       const ASubtitle   : string); override;
    class procedure GiveFeedback(const AWinControl : TWinControl); override;
    class function GetStyleURL(const AWinControl : TWinControl;
                                 var AStyleURL   : string): Boolean; override;
    class procedure SetStyleURL(const AWinControl : TWinControl;
                                const AStyleURL   : string); override;
    class function GetLatitude(const AWinControl : TWinControl;
                                 var ALatitude   : Double): Boolean; override;
    class procedure SetLatitude(const AWinControl : TWinControl;
                                      ALatitude   : Double); override;
    class function GetLongitude(const AWinControl : TWinControl;
                                  var ALongitude   : Double): Boolean; override;
    class procedure SetLongitude(const AWinControl : TWinControl;
                                       ALongitude   : Double); override;
    class function GetZoomLevel(const AWinControl : TWinControl;
                                  var AZoomLevel  : Double): Boolean; override;
    class procedure SetZoomLevel(const AWinControl : TWinControl;
                                       AZoomLevel  : Double); override;
    class function GetDirection(const AWinControl : TWinControl;
                                  var ADirection  : Double): Boolean; override;
    class procedure SetDirection(const AWinControl : TWinControl;
                                       ADirection  : Double); override;
    class function GetAllowZoom(const AWinControl : TWinControl;
                                  var AAllowZoom  : Boolean): Boolean; override;
    class procedure SetAllowZoom(const AWinControl : TWinControl;
                                       AAllowZoom  : Boolean); override;
    class function GetAllowScroll(const AWinControl  : TWinControl;
                                    var AAllowScroll : Boolean): Boolean; override;
    class procedure SetAllowScroll(const AWinControl  : TWinControl;
                                         AAllowScroll : Boolean); override;
    class function GetAllowRotate(const AWinControl  : TWinControl;
                                    var AAllowRotate : Boolean): Boolean; override;
    class procedure SetAllowRotate(const AWinControl  : TWinControl;
                                         AAllowRotate : Boolean); override;
    class function GetAllowTilt(const AWinControl : TWinControl;
                                  var AAllowTilt  : Boolean): Boolean; override;
    class procedure SetAllowTilt(const AWinControl : TWinControl;
                                       AAllowTilt  : Boolean); override;
    class function GetShowZoomControls(const AWinControl       : TWinControl;
                                         var AShowZoomControls : Boolean): Boolean; override;
    class procedure SetShowZoomControls(const AWinControl       : TWinControl;
                                              AShowZoomControls : Boolean); override;
    class function GetShowCompass(const AWinControl  : TWinControl;
                                    var AShowCompass : Boolean): Boolean; override;
    class procedure SetShowCompass(const AWinControl  : TWinControl;
                                         AShowCompass : Boolean); override;
    class procedure SetEnableClickMap(const AWinControl     : TWinControl;
                                            AEnableClickMap : Boolean); override;
    class procedure SetEnableDblClickMap(const AWinControl        : TWinControl;
                                               AEnableDblClickMap : Boolean); override;
    class procedure SetEnableRightClickMap(const AWinControl          : TWinControl;
                                                 AEnableRightClickMap : Boolean); override;
  end;


implementation

class function TCocoaWSMbMap.CreateHandle(const AWinControl : TWinControl;
                                          const AParams     : TCreateParams): TLCLIntfHandle;
var
  AMapView     : MGLMapView;
  AMapDelegate : MbMapDelegate;
begin
//  TCocoaWindowContent(AParams.WndParent).ownwin.setStyleMask(
//   TCocoaWindowContent(AParams.WndParent).ownwin.styleMask or (1 shl 15));  //layer-backed window
// note NSFullSizeContentViewWindowMask = 1 shl 15 in 10.10 NSWindow
// in Xcode, this is Full Size Content View check box for a Window in IB

  AMapView := MGLMapView(NSView(MGLMapView.alloc).lclInitWithCreateParams(AParams));

  AMapView.setWantsLayer(True);  //or logoView, attributionView, etc. won't show

  AMapDelegate := MbMapDelegate.alloc.initWithMap_winControl(AMapView, AWinControl);
  AMapView.setDelegate(AMapDelegate);

  Result := TLCLIntfHandle(AMapView);
end;


class procedure TCocoaWSMbMap.DestroyHandle(const AWinControl : TWinControl);
begin
  if not AWinControl.HandleAllocated then
    Exit;
  MGLMapView(AWinControl.Handle).delegate.release;
  NSObject(AWinControl.Handle).release;
end;


class function TCocoaWSMbMap.GetVersionNumber : Double;
begin
  Result := MapboxVersionNumber;
end;


class procedure TCocoaWSMbMap.SetAccessToken(const AAccessToken : string);
begin
  MGLAccountManager.setAccessToken(id(NSSTR(PAnsiChar(AAccessToken))));  //id cast workaround
end;


class procedure TCocoaWSMbMap.AddPointAnnotation(const AWinControl : TWinControl;
                                                       ALatitude   : Double;
                                                       ALongitude  : Double;
                                                 const ATitle      : string;
                                                 const ASubtitle   : string);
var
  geoLoc : CLLocationCoordinate2D;
  marker : MGLPointAnnotation;
begin
  geoLoc.latitude := ALatitude;
  geoLoc.longitude := ALongitude;
  marker := MGLPointAnnotation.alloc.init;
  marker.setCoordinate(geoLoc);
  marker.setTitle(id(NSSTR(PAnsiChar(ATitle))));        //id cast workarounds for
  marker.setSubtitle(id(NSSTR(PAnsiChar(ASubtitle))));  // CocoaAll/Foundation incompatibility
  MGLMapView(AWinControl.Handle).addAnnotation(marker);
  MGLMapView(AWinControl.Handle).selectAnnotation(marker);
end;


class procedure TCocoaWSMbMap.GiveFeedback(const AWinControl : TWinControl);
begin
  MGLMapView(AWinControl.Handle).giveFeedback(MGLMapView(AWinControl.Handle));  //not sure what to pass
end;


class function TCocoaWSMbMap.GetStyleURL(const AWinControl : TWinControl;
                                           var AStyleURL   : string): Boolean;
var
//  ANSURL : NSURL;
  ANSURL : id;  //workaround for incompatibility between CocoaAll.NSURL and Foundation.NSURL (Mapbox)
begin
  ANSURL := MGLMapView(AWinControl.Handle).styleURL;
  AStyleURL := ANSURL.absoluteString.UTF8String;  //is absoluteString right?
  Result := True;
end;


class procedure TCocoaWSMbMap.SetStyleURL(const AWinControl : TWinControl;
                                          const AStyleURL   : string);
var
//  ANSURL : NSURL;
  ANSURL : id;  //workaround for incompatibility between CocoaAll.NSURL and Foundation.NSURL (Mapbox)
begin
  ANSURL := nil;  {Use default style if blank URL string passed}
  if AStyleURL <> '' then
    ANSURL := NSURL.URLWithString(
               NSSTR(PAnsiChar(AStyleURL)).
                stringByAddingPercentEscapesUsingEncoding(NSASCIIStringEncoding));
  MGLMapView(AWinControl.Handle).setStyleURL(ANSURL);
end;


class function TCocoaWSMbMap.GetLatitude(const AWinControl : TWinControl;
                                           var ALatitude   : Double): Boolean;
begin
  ALatitude := MGLMapView(AWinControl.Handle).centerCoordinate.latitude;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetLatitude(const AWinControl : TWinControl;
                                                ALatitude   : Double);
var
  ACoord : CLLocationCoordinate2D;
begin
  ACoord := MGLMapView(AWinControl.Handle).centerCoordinate;
  ACoord.latitude := ALatitude;
  MGLMapView(AWinControl.Handle).setCenterCoordinate(ACoord);
end;


class function TCocoaWSMbMap.GetLongitude(const AWinControl : TWinControl;
                                            var ALongitude  : Double): Boolean;
begin
  ALongitude := MGLMapView(AWinControl.Handle).centerCoordinate.longitude;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetLongitude(const AWinControl : TWinControl;
                                                 ALongitude  : Double);
var
  ACoord : CLLocationCoordinate2D;
begin
  ACoord := MGLMapView(AWinControl.Handle).centerCoordinate;
  ACoord.longitude := ALongitude;
  MGLMapView(AWinControl.Handle).setCenterCoordinate(ACoord);
end;


class function TCocoaWSMbMap.GetZoomLevel(const AWinControl : TWinControl;
                                            var AZoomLevel  : Double): Boolean;
begin
  AZoomLevel := MGLMapView(AWinControl.Handle).zoomLevel;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetZoomLevel(const AWinControl : TWinControl;
                                                 AZoomLevel  : Double);
begin
  MGLMapView(AWinControl.Handle).setZoomLevel(AZoomLevel);
end;


class function TCocoaWSMbMap.GetDirection(const AWinControl : TWinControl;
                                            var ADirection  : Double): Boolean;
begin
  ADirection := MGLMapView(AWinControl.Handle).direction;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetDirection(const AWinControl : TWinControl;
                                                 ADirection  : Double);
begin
  MGLMapView(AWinControl.Handle).setDirection(ADirection);
end;


class function TCocoaWSMbMap.GetAllowZoom(const AWinControl : TWinControl;
                                            var AAllowZoom  : Boolean): Boolean;
begin
  AAllowZoom := MGLMapView(AWinControl.Handle).isZoomEnabled;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetAllowZoom(const AWinControl : TWinControl;
                                                 AAllowZoom  : Boolean);
begin
  MGLMapView(AWinControl.Handle).setZoomEnabled(AAllowZoom);
end;


class function TCocoaWSMbMap.GetAllowScroll(const AWinControl  : TWinControl;
                                              var AAllowScroll : Boolean): Boolean;
begin
  AAllowScroll := MGLMapView(AWinControl.Handle).isScrollEnabled;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetAllowScroll(const AWinControl  : TWinControl;
                                                   AAllowScroll : Boolean);
begin
  MGLMapView(AWinControl.Handle).setScrollEnabled(AAllowScroll);
end;


class function TCocoaWSMbMap.GetAllowRotate(const AWinControl  : TWinControl;
                                              var AAllowRotate : Boolean): Boolean;
begin
  AAllowRotate := MGLMapView(AWinControl.Handle).isRotateEnabled;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetAllowRotate(const AWinControl  : TWinControl;
                                                   AAllowRotate : Boolean);
begin
  MGLMapView(AWinControl.Handle).setRotateEnabled(AAllowRotate);
end;


class function TCocoaWSMbMap.GetAllowTilt(const AWinControl : TWinControl;
                                            var AAllowTilt  : Boolean): Boolean;
begin
  AAllowTilt := MGLMapView(AWinControl.Handle).isPitchEnabled;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetAllowTilt(const AWinControl : TWinControl;
                                                 AAllowTilt  : Boolean);
begin
  MGLMapView(AWinControl.Handle).setPitchEnabled(AAllowTilt);
end;


class function TCocoaWSMbMap.GetShowZoomControls(const AWinControl       : TWinControl;
                                                   var AShowZoomControls : Boolean): Boolean;
begin
  AShowZoomControls := not MGLMapView(AWinControl.Handle).zoomControls.isHidden;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetShowZoomControls(const AWinControl       : TWinControl;
                                                        AShowZoomControls : Boolean);
begin
  MGLMapView(AWinControl.Handle).zoomControls.setHidden(not AShowZoomControls);
end;


class function TCocoaWSMbMap.GetShowCompass(const AWinControl  : TWinControl;
                                              var AShowCompass : Boolean): Boolean;
begin
  AShowCompass := not MGLMapView(AWinControl.Handle).compass.isHidden;
  Result := True;
end;


class procedure TCocoaWSMbMap.SetShowCompass(const AWinControl  : TWinControl;
                                                   AShowCompass : Boolean);
begin
  MGLMapView(AWinControl.Handle).compass.setHidden(not AShowCompass);
end;


class procedure TCocoaWSMbMap.SetEnableClickMap(const AWinControl     : TWinControl;
                                                      AEnableClickMap : Boolean);
begin
  MbMapDelegate(MGLMapView(AWinControl.Handle).delegate).enableClick(AEnableClickMap);
end;


class procedure TCocoaWSMbMap.SetEnableDblClickMap(const AWinControl        : TWinControl;
                                                         AEnableDblClickMap : Boolean);
begin
  MbMapDelegate(MGLMapView(AWinControl.Handle).delegate).enableDblClick(AEnableDblClickMap);
end;


class procedure TCocoaWSMbMap.SetEnableRightClickMap(const AWinControl          : TWinControl;
                                                           AEnableRightClickMap : Boolean);
begin
  MbMapDelegate(MGLMapView(AWinControl.Handle).delegate).enableRightClick(AEnableRightClickMap);
end;


end.
