unit CocoaMbMapDelegate;

{
  LCL Mapbox map control.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).

  Mapbox macOS SDK: https://mapbox.github.io/mapbox-gl-native/macos
}

{$mode objfpc}{$H+}
{$modeswitch objectivec1}

interface

uses
  Foundation,
  AppKit,  //Mapbox can't use FPC's out-of-date CocoaAll
  CoreLocation,
  Mapbox,
  Controls,
  MbMap;

type
  MbMapDelegate = objcclass(NSObject, MGLMapViewDelegateProtocol,
                            NSGestureRecognizerDelegateProtocol)
  private
    mapView           : MGLMapView;
    mvWinControl      : TWinControl;
    clickGesture      : NSClickGestureRecognizer;
    dblClickGesture   : NSClickGestureRecognizer;
    rightClickGesture : NSClickGestureRecognizer;
  public
    function initWithMap_winControl(aMapView    : MGLMapView;
                                    aWinControl : TWinControl) : id; message 'initWithMap:winControl:';
    procedure dealloc; override;

    function clickIsEnabled : Boolean; message 'clickIsEnabled';
    procedure enableClick(aEnableClick : Boolean); message 'enableClick:';
    procedure handleClick(aGesture : NSClickGestureRecognizer); message 'handleClick:';

    function dblClickIsEnabled : Boolean; message 'dblClickIsEnabled';
    procedure enableDblClick(aEnableDblClick : Boolean); message 'enableDblClick:';
    procedure handleDblClick(aGesture : NSClickGestureRecognizer); message 'handleDblClick:';

    function rightClickIsEnabled : Boolean; message 'rightClickIsEnabled';
    procedure enableRightClick(aEnableRightClick : Boolean); message 'enableRightClick:';
    procedure handleRightClick(aGesture : NSClickGestureRecognizer); message 'handleRightClick:';

    function mapView_annotationCanShowCallout(
              aMapView   : MGLMapView; 
              annotation : MGLAnnotationProtocol): Boolean; message 'mapView:annotationCanShowCallout:';
    procedure mapView_didDeselectAnnotation(
               aMapView   : MGLMapView;
               annotation : MGLAnnotationProtocol); message 'mapView:didDeselectAnnotation:';
  end;


implementation

type
  TIntCustomMbMap = class(TCustomMbMap);  //so we can access protected methods


function MbMapDelegate.initWithMap_winControl(aMapView    : MGLMapView;
                                              aWinControl : TWinControl) : id;
begin
  Result := init;

  mapView := aMapView;
  mvWinControl := aWinControl;

  clickGesture := NSClickGestureRecognizer.alloc.initWithTarget_action(
                   self, objcselector('handleClick:'));
  clickGesture.setDelegate(self);

  dblClickGesture := NSClickGestureRecognizer.alloc.initWithTarget_action(
                      self, objcselector('handleDblClick:'));
  dblClickGesture.setDelegate(self);
  dblClickGesture.setNumberOfClicksRequired(2);

  rightClickGesture := NSClickGestureRecognizer.alloc.initWithTarget_action(
                        self, objcselector('handleRightClick:'));
  rightClickGesture.setDelegate(self);
  rightClickGesture.setButtonMask(2);
end;


procedure MbMapDelegate.dealloc;
begin
  clickGesture.release;
  dblClickGesture.release;
  rightClickGesture.release;
  inherited dealloc;
end;


function MbMapDelegate.clickIsEnabled : Boolean;
begin
  Result := mapView.gestureRecognizers.containsObject(clickGesture);
end;


procedure MbMapDelegate.enableClick(aEnableClick : Boolean);
begin
  if aEnableClick then
    begin
    if not clickIsEnabled then
      mapView.addGestureRecognizer(clickGesture);
    end
  else  {Disable and revert back to map's default click gesture recognizer}
    begin
    if clickIsEnabled then
      mapView.removeGestureRecognizer(clickGesture);
    end;
end;


procedure MbMapDelegate.handleClick(aGesture : NSClickGestureRecognizer);
var
  viewLoc : NSPoint;
  geoLoc  : CLLocationCoordinate2D;
begin
  viewLoc := aGesture.locationInView(mapView);
  geoLoc := mapView.convertPoint_toCoordinateFromView(viewLoc, mapView);
  TIntCustomMbMap(mvWinControl).DoOnClickMap(geoLoc.latitude, geoLoc.Longitude);
end;


function MbMapDelegate.dblClickIsEnabled : Boolean;
begin
  Result := mapView.gestureRecognizers.containsObject(dblClickGesture);
end;


procedure MbMapDelegate.enableDblClick(aEnableDblClick : Boolean);
begin
  if aEnableDblClick then
    begin
    if not dblClickIsEnabled then
      mapView.addGestureRecognizer(dblClickGesture);
    end
  else  {Disable and revert back to map's default double-click gesture recognizer}
    begin
    if dblClickIsEnabled then
      mapView.removeGestureRecognizer(dblClickGesture);
    end;
end;


procedure MbMapDelegate.handleDblClick(aGesture : NSClickGestureRecognizer);
var
  viewLoc : NSPoint;
  geoLoc  : CLLocationCoordinate2D;
begin
  viewLoc := aGesture.locationInView(mapView);
  geoLoc := mapView.convertPoint_toCoordinateFromView(viewLoc, mapView);
  TIntCustomMbMap(mvWinControl).DoOnDblClickMap(geoLoc.latitude, geoLoc.Longitude);
end;


function MbMapDelegate.rightClickIsEnabled : Boolean;
begin
  Result := mapView.gestureRecognizers.containsObject(rightClickGesture);
end;


procedure MbMapDelegate.enableRightClick(aEnableRightClick : Boolean);
begin
  if aEnableRightClick then
    begin
    if not rightClickIsEnabled then
      mapView.addGestureRecognizer(rightClickGesture);
    end
  else  {Disable and revert back to map's default double-click gesture recognizer}
    begin
    if rightClickIsEnabled then
      mapView.removeGestureRecognizer(rightClickGesture);
    end;
end;


procedure MbMapDelegate.handleRightClick(aGesture : NSClickGestureRecognizer);
var
  viewLoc : NSPoint;
  geoLoc  : CLLocationCoordinate2D;
begin
  viewLoc := aGesture.locationInView(mapView);
  geoLoc := mapView.convertPoint_toCoordinateFromView(viewLoc, mapView);
  TIntCustomMbMap(mvWinControl).DoOnRightClickMap(geoLoc.latitude, geoLoc.Longitude);
end;


function MbMapDelegate.mapView_annotationCanShowCallout(
                         aMapView   : MGLMapView; 
                         annotation : MGLAnnotationProtocol): Boolean;
begin
  Result := True;
end;


procedure MbMapDelegate.mapView_didDeselectAnnotation(
                          aMapView   : MGLMapView;
                          annotation : MGLAnnotationProtocol);
begin
  aMapView.removeAnnotation(annotation);
  annotation.release;
end;


end.
