unit MbMap;

{
  LCL Mapbox map control.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).
}

{$mode objfpc}{$H+}

interface

uses
  SysUtils,
  Classes,
  LResources,
  Controls;

type
  TMbClickMapEvent = procedure(Sender    : TObject;
                               Latitude  : Double;
                               Longitude : Double) of object;


  TCustomMbMap = class(TWinControl)
  private
    FStyleURL         : string;
    FLatitude         : Double;
    FLongitude        : Double;
    FZoomLevel        : Double;
    FDirection        : Double;
    FAllowZoom        : Boolean;
    FAllowScroll      : Boolean;
    FAllowRotate      : Boolean;
    FAllowTilt        : Boolean;
    FShowZoomControls : Boolean;
    FShowCompass      : Boolean;
    FOnClickMap       : TMbClickMapEvent;
    FOnDblClickMap    : TMbClickMapEvent;
    FOnRightClickMap  : TMbClickMapEvent;
    UseLiveMap        : Boolean;
  protected
    class procedure WSRegisterClass; override;
    procedure CreateWnd; override;
    procedure SetOnClickMap(AOnClickMap : TMbClickMapEvent); virtual;
    procedure SetOnDblClickMap(AOnDblClickMap : TMbClickMapEvent); virtual;
    procedure SetOnRightClickMap(AOnRightClickMap : TMbClickMapEvent); virtual;
    function GetStyleURL : string; virtual;
    procedure SetStyleURL(const AStyleURL : string); virtual;
    function GetLatitude : Double; virtual;
    procedure SetLatitude(ALatitude : Double); virtual;
    function GetLongitude : Double; virtual;
    procedure SetLongitude(ALongitude : Double); virtual;
    function GetZoomLevel : Double; virtual;
    procedure SetZoomLevel(AZoomLevel : Double); virtual;
    function GetDirection : Double; virtual;
    procedure SetDirection(ADirection : Double); virtual;
    function GetAllowZoom : Boolean; virtual;
    procedure SetAllowZoom(AAllowZoom : Boolean); virtual;
    function GetAllowScroll : Boolean; virtual;
    procedure SetAllowScroll(AAllowScroll : Boolean); virtual;
    function GetAllowRotate : Boolean; virtual;
    procedure SetAllowRotate(AAllowRotate : Boolean); virtual;
    function GetAllowTilt : Boolean; virtual;
    procedure SetAllowTilt(AAllowTilt : Boolean); virtual;
    function GetShowZoomControls : Boolean; virtual;
    procedure SetShowZoomControls(AShowZoomControls : Boolean); virtual;
    function GetShowCompass : Boolean; virtual;
    procedure SetShowCompass(AShowCompass : Boolean); virtual;
    procedure DoOnClickMap(ALatitude  : Double;
                           ALongitude : Double);
    procedure DoOnDblClickMap(ALatitude  : Double;
                              ALongitude : Double);
    procedure DoOnRightClickMap(ALatitude  : Double;
                                ALongitude : Double);
  public
    constructor Create(AOwner : TComponent); override;

    class function GetVersionNumber : Double;
    class procedure SetAccessToken(const AAccessToken : string);

    procedure AddPointAnnotation(      ALatitude  : Double;
                                       ALongitude : Double;
                                 const ATitle     : string;
                                 const ASubtitle  : string);
    procedure GiveFeedback;

    property StyleURL         : string read GetStyleURL write SetStyleURL;
    property Latitude         : Double read GetLatitude write SetLatitude;
    property Longitude        : Double read GetLongitude write SetLongitude;
    property ZoomLevel        : Double read GetZoomLevel write SetZoomLevel;
    property Direction        : Double read GetDirection write SetDirection;
    property AllowZoom        : Boolean read GetAllowZoom write SetAllowZoom;
    property AllowScroll      : Boolean read GetAllowScroll write SetAllowScroll;
    property AllowRotate      : Boolean read GetAllowRotate write SetAllowRotate;
    property AllowTilt        : Boolean read GetAllowTilt write SetAllowTilt;
    property ShowZoomControls : Boolean read GetShowZoomControls write SetShowZoomControls;
    property ShowCompass      : Boolean read GetShowCompass write SetShowCompass;
    property OnClickMap       : TMbClickMapEvent read FOnClickMap write SetOnClickMap;
    property OnDblClickMap    : TMbClickMapEvent read FOnDblClickMap write SetOnDblClickMap;
    property OnRightClickMap  : TMbClickMapEvent read FOnRightClickMap write SetOnRightClickMap;
  end;


  TMbMap = class(TCustomMbMap)
  published
    property Align;
    property AllowRotate;
    property AllowScroll;
    property AllowTilt;
    property AllowZoom;
    property Anchors;
    property Constraints;
    property Direction;
    property Latitude;
    property Longitude;
    property OnClickMap;
    property OnDblClickMap;
    property OnRightClickMap;
    property ShowCompass;
    property ShowZoomControls;
    property StyleURL;
    property ZoomLevel;
  end;


procedure Register;


implementation

uses
  Mapbox_Ctrls_Factory,
  WSMbMap;


constructor TCustomMbMap.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  ControlStyle := ControlStyle + [csFramed];

  Height := 140;
  Width := 200;
  TabStop := False;

  FStyleURL := '';  //use default style
  FLatitude := 0;
  FLongitude := 0;
  FZoomLevel := 0;  //0="each tile covers the entire world map"
  FDirection := 0;  //0="top edge of the map view corresponds to true north"
  FAllowZoom := True;  //current Mapbox defaults
  FAllowScroll := True;
  FAllowRotate := True;  //on Mac, option key + mouse left-right
  FAllowTilt := True;    //on Mac, option key + mouse up-down
  FShowZoomControls := True;
  FShowCompass := True;
  FOnClickMap := nil;      //by default, use map's built-in click handlers
  FOnDblClickMap := nil;   // (double-click on map to zoom, single-click on
  FOnRightClickMap := nil; // zoom controls); setting disables built-in handler

  UseLiveMap := not (csDesigning in ComponentState);
   //Some side effects with properties when use live map in design mode,
   // so for now just ignore any user manipulation of map when designing.
   //TODO: investigate whether live map in design mode is viable.
end;


class procedure TCustomMbMap.WSRegisterClass;  
begin
  inherited;
  WSRegisterMbMap;
end;


procedure TCustomMbMap.CreateWnd;
begin
  inherited CreateWnd;
   {Handle created, so okay to finish initializing properties}
  SetStyleURL(FStyleURL);
  SetLatitude(FLatitude);
  SetLongitude(FLongitude);
  SetZoomLevel(FZoomLevel);
  SetDirection(FDirection);
  SetAllowZoom(FAllowZoom);
  SetAllowScroll(FAllowScroll);
  SetAllowRotate(FAllowRotate);
  SetAllowTilt(FAllowTilt);
  SetShowZoomControls(FShowZoomControls);
  SetShowCompass(FShowCompass);
  SetOnClickMap(FOnClickMap);
  SetOnDblClickMap(FOnDblClickMap);
  SetOnRightClickMap(FOnRightClickMap);
end;


class function TCustomMbMap.GetVersionNumber : Double;
begin  //class method provided as a convenience for getting Mapbox version
  Result := GetMbVersionNumber;
end;


class procedure TCustomMbMap.SetAccessToken(const AAccessToken : string);
begin  //can call this class method before creating TMbMap or, on Mac, set
       // MGLMapboxAccessToken key to access token in app's Info.plist
  SetMbAccessToken(AAccessToken);
end;


procedure TCustomMbMap.AddPointAnnotation(      ALatitude  : Double;
                                                ALongitude : Double;
                                          const ATitle     : string;
                                          const ASubtitle  : string);
begin
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).AddPointAnnotation(self, ALatitude, ALongitude,
                                                     ATitle, ASubtitle);
end;


procedure TCustomMbMap.GiveFeedback;
begin
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).GiveFeedback(self);
end;


function TCustomMbMap.GetStyleURL : string;
begin
  Result := FStyleURL;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetStyleURL(self, Result) then
      FStyleURL := Result;
    end;
end;


procedure TCustomMbMap.SetStyleURL(const AStyleURL : string);
begin
  FStyleURL := AStyleURL;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetStyleURL(self, AStyleURL);
end;


function TCustomMbMap.GetLatitude : Double;
begin
  Result := FLatitude;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetLatitude(self, Result) then
      FLatitude := Result;
    end;
end;


procedure TCustomMbMap.SetLatitude(ALatitude : Double);
begin
  FLatitude := ALatitude;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetLatitude(self, ALatitude);
end;


function TCustomMbMap.GetLongitude : Double;
begin
  Result := FLongitude;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetLongitude(self, Result) then
      FLongitude := Result;
    end;
end;


procedure TCustomMbMap.SetLongitude(ALongitude : Double);
begin
  FLongitude := ALongitude;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetLongitude(self, ALongitude);
end;


function TCustomMbMap.GetZoomLevel : Double;
begin
  Result := FZoomLevel;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetZoomLevel(self, Result) then
      FZoomLevel := Result;
    end;
end;


procedure TCustomMbMap.SetZoomLevel(AZoomLevel : Double);
begin  //note can't be negative; zoom level 0 is entire world
  FZoomLevel := AZoomLevel;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetZoomLevel(self, AZoomLevel);
end;


function TCustomMbMap.GetDirection : Double;
begin
  Result := FDirection;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetDirection(self, Result) then
      FDirection := Result;
    end;
end;


procedure TCustomMbMap.SetDirection(ADirection : Double);
begin  //note can't be negative; 0=true north
  FDirection := ADirection;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetDirection(self, ADirection);
end;


function TCustomMbMap.GetAllowZoom : Boolean;
begin
  Result := FAllowZoom;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetAllowZoom(self, Result) then
      FAllowZoom := Result;
    end;
end;


procedure TCustomMbMap.SetAllowZoom(AAllowZoom : Boolean);
begin
  FAllowZoom := AAllowZoom;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetAllowZoom(self, AAllowZoom);
end;


function TCustomMbMap.GetAllowScroll : Boolean;
begin
  Result := FAllowScroll;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetAllowScroll(self, Result) then
      FAllowScroll := Result;
    end;
end;


procedure TCustomMbMap.SetAllowScroll(AAllowScroll : Boolean);
begin
  FAllowScroll := AAllowScroll;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetAllowScroll(self, AAllowScroll);
end;


function TCustomMbMap.GetAllowRotate : Boolean;
begin
  Result := FAllowRotate;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetAllowRotate(self, Result) then
      FAllowRotate := Result;
    end;
end;


procedure TCustomMbMap.SetAllowRotate(AAllowRotate : Boolean);
begin
  FAllowRotate := AAllowRotate;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetAllowRotate(self, AAllowRotate);
end;


function TCustomMbMap.GetAllowTilt : Boolean;
begin
  Result := FAllowTilt;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetAllowTilt(self, Result) then
      FAllowTilt := Result;
    end;
end;


procedure TCustomMbMap.SetAllowTilt(AAllowTilt : Boolean);
begin
  FAllowTilt := AAllowTilt;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetAllowTilt(self, AAllowTilt);
end;


function TCustomMbMap.GetShowZoomControls : Boolean;
begin
  Result := FShowZoomControls;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetShowZoomControls(self, Result) then
      FShowZoomControls := Result;
    end;
end;


procedure TCustomMbMap.SetShowZoomControls(AShowZoomControls : Boolean);
begin
  FShowZoomControls := AShowZoomControls;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetShowZoomControls(self, AShowZoomControls);
end;


function TCustomMbMap.GetShowCompass : Boolean;
begin
  Result := FShowCompass;
  if HandleAllocated and UseLiveMap then
    begin
    if TWSMbMapClass(WidgetSetClass).GetShowCompass(self, Result) then
      FShowCompass := Result;
    end;
end;


procedure TCustomMbMap.SetShowCompass(AShowCompass : Boolean);
begin
  FShowCompass := AShowCompass;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetShowCompass(self, AShowCompass);
end;


procedure TCustomMbMap.SetOnClickMap(AOnClickMap : TMbClickMapEvent);
begin
  FOnClickMap := AOnClickMap;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetEnableClickMap(self, Assigned(AOnClickMap));
end;


procedure TCustomMbMap.SetOnDblClickMap(AOnDblClickMap : TMbClickMapEvent);
begin
  FOnDblClickMap := AOnDblClickMap;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetEnableDblClickMap(self, Assigned(AOnDblClickMap));
end;


procedure TCustomMbMap.SetOnRightClickMap(AOnRightClickMap : TMbClickMapEvent);
begin
  FOnRightClickMap := AOnRightClickMap;
  if HandleAllocated and UseLiveMap then
    TWSMbMapClass(WidgetSetClass).SetEnableRightClickMap(self, Assigned(AOnRightClickMap));
end;


procedure TCustomMbMap.DoOnClickMap(ALatitude  : Double;
                                    ALongitude : Double);
begin
  if Assigned(FOnClickMap) then
    FOnClickMap(self, ALatitude, ALongitude);
end;


procedure TCustomMbMap.DoOnDblClickMap(ALatitude  : Double;
                                       ALongitude : Double);
begin
  if Assigned(FOnDblClickMap) then
    FOnDblClickMap(self, ALatitude, ALongitude);
end;


procedure TCustomMbMap.DoOnRightClickMap(ALatitude  : Double;
                                         ALongitude : Double);
begin
  if Assigned(FOnRightClickMap) then
    FOnRightClickMap(self, ALatitude, ALongitude);
end;



procedure Register;
begin
  RegisterComponents('Mapbox', [TMbMap]);
end;


initialization
{$I MbMap.lrs}  //glyph courtesy of https://openclipart.org/detail/3913/earth

end.
